/*
 * =====================================================================================
 *
 *       Filename:  stats.h
 *
 *    Description:  simple distribution execise
 *
 *        Version:  1.0
 *        Created:  04/15/2011 06:12:42 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Ruben Safir, 
 *        Company:  
 *
 * =====================================================================================
 */
#ifndef STATS_H
#define STATS_H
#include	<iostream>
#include	<fstream>
#include 	<climits>
#include 	<cmath>
#include	<stdlib.h>
#include        <iomanip>
#include	"linklist.h"

/*
 * =====================================================================================
 *        Class:  Distribution
 *  Description:  Keeps Track of Distribution of 6's (or anything else) in a series of List
 * =====================================================================================
 */


namespace stats{


   template<class T>
      class Distribution {
	 template<T> friend std::ostream & operator<<(std::ostream &, const Distribution<T>&);
	 public:
	 /* ====================  LIFECYCLE     ======================================= */
	 Distribution (T descr, int occurances = 0);
	 //Distribution():freq(NULL), occurances(0){};
	 Distribution(){};
	 //	Distribution ( const Distribution &other );   /* copy constructor */
	 //		~Distribution ();                            /* destructor       */
	 /* ====================  ACCESSORS     ======================================= */
	 T description()const{ return freq;}
	 int population()const { return occurances; }	
	 /* ====================  MUTATORS      ======================================= */
	 void increase_occ(){ ++occurances; std::cout << "description " << freq << " occurances " << occurances << std::endl; }
	 void descrease_occ(){ --occurances; }
	 /* ====================  OPERATORS     ======================================= */
	 //Distribution& operator = ( const Distribution &other ); /* assignment operator */
	 T operator()(){
	    return freq;
	 }


	 bool operator==(Distribution &tmp){
	    if(this->freq == tmp.freq)
	       return true;
	    return false;
	 }


	 bool operator<(Distribution &tmp){
	    if(freq < tmp.freq)
	       return true;
	    return false;
	 }
	 chainlist::List< stats::Distribution<T> > * tally;  //a list of distribution talleys 

	 float stddev(chainlist::List<stats::Distribution<T> > *);




	 protected:
	 /* ====================  DATA MEMBERS  ======================================= */
	 private:
	 /* ====================  DATA MEMBERS  ======================================= */
	 T freq;  //description of unique identifier of a sample point in a List
	 int occurances; //description of how many times a frequency was found in a list
      }; /* -----  end of class Distribution  ----- */


   template<typename T>
      std::ostream & operator << ( std::ostream & os, const Distribution<T> & obj )
      {
	 T desc = obj.description();
	 int pop  = obj.population();
	 os << "The Identification of " << desc << " was seen " << pop ;
	 return os;
      }		/* -----  end of function operator <<  ----- */

   template<typename T>
      Distribution<T>::Distribution(T descr, int occ): occurances(occ){
	 freq = descr;
      }

   
   // routinines not part of the distribution class

   /* Routine to determin the mean value of populations in a list of distribribution members  */
   template <typename T>
      float mean_list(chainlist::List< Distribution<T> > * tally);

   /*  Routine to go though a single list and add it to an existing distribution table */
   template<typename T>
      void mount_individual_data_point(chainlist::List<T> * tabulate, chainlist::List<stats::Distribution<T> > * table);

   /* Routine to find all the occurances of a type in a list of lists */
   template<typename T>
      void take_tally(chainlist::List<T> *,chainlist::List<stats::Distribution<T> > *);


   //calculation standard deviation of distribution list
   //
   template<typename T>
      float stddev(chainlist::List<stats::Distribution<T> > * tally){
	 float dev, vari = 0;
	 //walk through the list
	 if( tally->endd() == 0 ){
	    std::cout << "Empty List"  << std::endl;
	    exit(0);
	 }
	 float mean = mean_list(tally);
	 tally->cursor() = tally->front();

	 while(tally->cursor() != tally->endd() ){
	    vari += pow (tally->cursor()->value()->population() -  mean, 2 );
	    tally->cursor() = tally->cursor()->next();
	 }
	 vari += pow (tally->cursor()->value()->population() -  mean, 2 );
	 dev = sqrt( (vari / tally->size()) ); 





	 return dev;
      }



   template<typename T>
      float stddev(chainlist::List< chainlist::List<stats::Distribution<T> > * > *  tallies, T search_val){
	 float dev, vari = 0;
	 //walk through the list
	 chainlist::List<stats::Distribution<T> > * dump;
	 if( tallies->endd() == 0 ){
	    std::cout << "Empty List"  << std::endl;
	    exit(0);
	 }
	 float mean = mean_list(tallies, search_val);
	 tallies->cursor() = tallies->front();
	 while(tallies->cursor() != tallies->endd() ){
	    dump = *(tallies->cursor()->value());
	    dump->cursor() = dump->front(); //set cursor to the front 
	    if(dump->endd() == 0){ 
	       std::cout << "Empty Distribution List"  << std::endl;
	       exit(0);
	    }
	    //we don't need a second while loop here because each Distribution object in the list is unique
	    dump->find_value(search_val);
	    if(dump->cursor() != 0)
	       vari += pow(dump->cursor()->value()->population() - mean, 2);
	    else
	       vari += pow(0 - mean, 2);

	    tallies->cursor() = tallies->cursor()->next();
	 }
	 dump->find_value(search_val);
	 if(dump->cursor() != 0)
	    vari += pow(dump->cursor()->value()->population() - mean, 2);
	 else
	    vari += pow(0 - mean, 2);
	 dev = sqrt( (vari / tallies->size()) );
	 return dev; 
      
      
      }




   template<typename T>
      void take_tally(chainlist::List<T> * tabulate, chainlist::List<stats::Distribution<T> > * table){
	 for(tabulate->cursor()=tabulate->front();tabulate->cursor() != tabulate->endd(); tabulate->cursor( tabulate->cursor()->next() ) ){ //build distribution list
	    mount_individual_data_point(tabulate, table);
	 }
	 //we are at the end of tabulate
	 mount_individual_data_point(tabulate, table);
	 table->sort(*table);
      }

   template<typename T>
      void mount_individual_data_point(chainlist::List<T> * tabulate, chainlist::List<stats::Distribution<T> > * table){
	 T val;
	 stats::Distribution<T> * j;
	 val = *(tabulate->cursor()->value()); //get a value
	 table->cursor()= table->front(); //check to see if the distribution list exists
	 if(!table->cursor()){ // if not add a distribution table to the List of distributions
	    j = new stats::Distribution<T> (val);
	    table->insert(*j ); //now we have at least one
	    delete j;
	    j=table->cursor()->value();//and increased its population
	    j->increase_occ();
	 }else{
	    //otherwise search for a distribution node described as value
	    table->find_value(val); 
	    if( table->cursor() ){
	       j=table->cursor()->value();//and increase its population
	       j->increase_occ();
	    }else{//otherwise add a new node
	       j = new stats::Distribution<T> (val);
	       table->insert( *j ); //now we have one for that value
	       delete j;
	       j=table->cursor()->value();//and increased its population
	       j->increase_occ();
	    }
	 }
      }



   template<typename T>
      float mean_list(chainlist::List< chainlist::List<stats::Distribution<T> >*  > *  tallies, T search_val){
	 if(tallies->endd() == 0){
	    std::cout << "Empty List" << std::endl;
	    return 0.0;
	 }
	 int sum = 0;
	 chainlist::List<stats::Distribution<T> > * dump;
	 tallies->cursor() = tallies->front();
	 while(tallies->cursor() != tallies->endd() ){
		 //tallies->cursor()->value()->find_value(search_val);
		 dump = *(tallies->cursor()->value());
		 //dump->cursor() = dump->endd();
		 //std::cout << "Testing\n" << *(dump->cursor()->value()) << std::endl;
		 dump->find_value(search_val);
		 if(dump->cursor() != NULL)
		    sum +=  dump->cursor()->value()->population();
		 tallies->cursor(tallies->cursor()->next());
	 }
	 dump = *(tallies->cursor()->value());
	 dump->find_value(search_val);
	 if(dump->cursor() != 0)
	    sum +=  dump->cursor()->value()->population();
	 float tot = (float) sum/(float)(tallies->size());
	 std::cout << "Mean " << tot << " sum " << sum << " size " << tallies->size() << std::endl;
	 return tot;

      }

   template <typename T>
      float mean_list(chainlist::List< Distribution<T> > * tally){
	 if(tally->endd() == 0){
	    std::cout << "Empty List" << std::endl;
	    return 0.0;
	 }

	 int sum = 0;

	 tally->cursor() = tally->front();
	 while(tally->cursor() != tally->endd() ){
	    sum +=  tally->cursor()->value()->population() ;
	    tally->cursor(tally->cursor()->next());
	 }
	 sum +=  tally->cursor()->value()->population() ;

	float tot = (float)sum/(float)(tally->size());
	std::cout << "Mean " << tot << " sum " << sum << " size " << tally->size() << std::endl;

	return tot;
      }





}
#endif /* STATS_H */
